# Copyright (C) 2005, 2006, 2008 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Topological sorting routines."""

from .known_graph import KnownGraph

__all__ = ["MergeSorter", "TopoSorter", "merge_sort", "topo_sort"]

# The Rust implementations are optional
from ._graph_rs import MergeSorter, TopoSorter, merge_sort


def topo_sort(graph):
    """Topological sort a graph.

    graph -- sequence of pairs of node->parents_list.

    The result is a list of node names, such that all parents come before their
    children.

    node identifiers can be any hashable object, and are typically strings.

    This function has the same purpose as the TopoSorter class, but uses a
    different algorithm to sort the graph. That means that while both return a
    list with parents before their child nodes, the exact ordering can be
    different.

    topo_sort is faster when the whole list is needed, while when iterating
    over a part of the list, TopoSorter.iter_topo_order should be used.
    """
    kg = KnownGraph(dict(graph))
    return kg.topo_sort()



# Copyright (C) 2025 Breezy Developers
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import os
import shutil
import tempfile
import unittest
from unittest import TestCase

__all__ = ["TestCaseInTempDir"]


class TestCaseInTempDir(TestCase):
    """Minimal TestCase that runs in a temporary directory.

    Only implements the functionality actually needed by vcsgraph tests.
    """

    def setUp(self):
        super().setUp()
        self.test_dir = tempfile.mkdtemp(prefix="vcsgraph_test_")
        self.original_dir = os.getcwd()
        os.chdir(self.test_dir)

    def tearDown(self):
        os.chdir(self.original_dir)
        shutil.rmtree(self.test_dir)
        super().tearDown()

    def assertPathExists(self, path):
        """Fail unless path exists."""
        self.assertTrue(os.path.lexists(path), f"{path} does not exist")

    def assertPathDoesNotExist(self, path):
        """Fail if path exists."""
        self.assertFalse(os.path.lexists(path), f"{path} exists")


def test_suite() -> unittest.TestSuite:
    names = [
        "known_graph",
        "graph",
        "tsort",
    ]
    module_names = ["vcsgraph.tests.test_" + name for name in names]
    loader = unittest.TestLoader()
    return loader.loadTestsFromNames(module_names)



# Copyright (C) 2005-2010 Canonical Ltd
# Copyright (C) 2018-2025 Breezy Developers
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Graph algorithms for version control systems.

This package contains graph-related algorithms that are useful for
version control systems, including:

- Topological sorting (tsort)
- Graph operations (graph)
"""

__all__ = [
    "BaseVersionedFile",
    "DictParentsProvider",
    "FrozenHeadsCache",
    "Graph",
    "KnownGraph",
    "MultiMemoryVersionedFile",
    "MultiVersionedFile",
    "invert_parent_map",
    "topo_sort",
]

__version__ = (0, 3, 2)

# Re-export commonly used functions and classes
from .graph import (
    DictParentsProvider,
    FrozenHeadsCache,
    Graph,
    invert_parent_map,
)
from .known_graph import KnownGraph
from .tsort import topo_sort
